/*
 * @Author Vitim
 */

var Battery=new function(){

    /*Public properties*/
	this.level;                 //readonly public property
	this.remaingTime;           //readonly public property
	this.charging;             	//readonly public property
	this.powerCordConnected;   	//readonly public property

    this.refreshInterval=1000;      //miliseconds

    /*Private properties*/
    var parent=this;
    var Timer;


    /*Public Methods*/
	this.Create = function(){
	
		//Add Event on PowerLineStatusChanged (Charger plugged/unplugged)
		System.Machine.PowerStatus.powerLineStatusChanged = parent.PowerCordChanged;
		
		//Add Timer to refresh
        Timer=setInterval(this.Refresh,this.refreshInterval);
		
		this.Refresh();
		
		Update.check(); //check for update
		window.setTimeout(Update.check,1000*60); //check for update again after delay
		
	}
	
	
	this.Destroy=function(){
		
		this.level=-1; //force update on next refresh
		this.powerCordConnected=(!this.powerCordConnected);
		
        clearInterval(Timer);
		System.Machine.PowerStatus.powerLineStatusChanged=null;
		Sound.Stop();
	}
	
	this.Refresh=function(){
		
		//store previous state
		var previousLevel=parent.level;
		var previousRemaingTime=parent.remaingTime;
		
		parent.level=System.Machine.PowerStatus.batteryPercentRemaining;
		parent.remaingTime=System.Machine.PowerStatus.batteryCapacityRemaining;

		Remaing.feed(parent.level, parent.charging, parent.powerCordConnected);

		
		if(previousLevel!=parent.level || previousRemaingTime!=parent.remaingTime){ //verify changes to reduce CPU load, preventing unecessary viewcontroler calls
			// Make changes on ViewController
			ViewController.Level(parent.level);
			//ViewController.Time(parent.remaingTime);
		}
		
		parent.PowerCordChanged();
		
	}
	
	
	this.PowerCordChanged=function(){ //function called when (Charger plugged/unplugged)
		//store previous state
		var previousPowerCordConnected=parent.powerCordConnected;
		var previousCharging=parent.charging;
		
		//update state
		parent.powerCordConnected = System.Machine.PowerStatus.isPowerLineConnected;
		parent.charging = System.Machine.PowerStatus.isBatteryCharging;
		
		if(previousPowerCordConnected!=parent.powerCordConnected || previousCharging!=parent.charging){	//verify changes to reduce CPU load, preventing unecessary viewcontroler calls
			
			// Make change on ViewController
			ViewController.Status(parent.powerCordConnected,parent.charging);
			
			Remaing.show=(!(parent.powerCordConnected || parent.charging) ); //Hide remaing time on AC power
			ViewController.Time(Remaing.seconds, Remaing.show); //update
			
		}
		
		if(parent.powerCordConnected!=previousPowerCordConnected){
			parent.SoundEvent();
		}
		
		//Add Event on PowerLineStatusChanged (Charger plugged/unplugged)
		System.Machine.PowerStatus.powerLineStatusChanged = parent.PowerCordChanged;
	}
	
	
	this.SoundEvent=function(){
		
		if(parent.powerCordConnected==true || parent.charging==true){
			Sound.Play("Plugged");
		}else{
			Sound.Play("Unplugged");
		}
	}
}

/**
Play a sound file (.wav)
*/
var Sound=new function(){

	this.enabled=true;	//false to Mute
	
	var themeName;
	var path;
	
	this.SetTheme=function(name){
		themeName=name;
		path=System.Gadget.path+"\\Resources\\sounds\\"+themeName+"\\";
	}
	
	this.Play=function(file){
		
		if(this.enabled==true){
			System.Sound.playSound(path + file + ".wav");		
		}
	}
	
	this.Stop=function(){
		System.Sound.playSound('');
	}
}





var Remaing = new function(){
	
	this.seconds=0;
	this.show=false;
	
	//algorithm settings
	var range=5; //calc base on last 5 percents
	var minrange=3; //only calc after this number of percents
	var critical=5; //critical battery level
	
	var Percents=[];
	var Times=[];
	
	var remaingSeconds;
	var nativeRemaing;
	

	
	this.feed=function(percent,isCharging,onPower){
		
		if(isCharging==false && onPower==false){
			
			this.show=true; //not connected to ac power, show remaing time
			
			if(checkNativeRemaing()){
				var previousNativeRemaing=nativeRemaing
				nativeRemaing=System.Machine.PowerStatus.batteryCapacityRemaining;
				
				if(previousNativeRemaing!=nativeRemaing){ //changed
					
					ViewController.Time(nativeRemaing,true);
					this.seconds=nativeRemaing;
					
				}
			}
			
			else //native not supported, using algorithm 
			
			if(percent!=Percents[Percents.length-1]){ //percent changed
				
				if(Percents.length>0 && percent>Percents[Percents.length-1]){ //battery has been charged
					this.reset();
				}
				
				Percents[Percents.length]=percent;
				Times[Times.length]=currentTime();
				
				while(Percents.length>range){
					Percents.shift();
					Times.shift();
				}
				
				this.calc();
			}
		}
		
		else{ //conected to ac power, hide the remaing time
		
			if(this.show==true){ //reduce cpu overload.
				ViewController.Time(0,false);
				this.show=false;
				
				this.reset();
			}
		}
	}
	
	this.calc=function(){
		
		if(Times.length>=minrange){
		
			var lastTime=Times[Times.length-1]-Times[Times.length-2];
			
			var average=calcAvgDiference(Times);
			var average=(lastTime+lastTime+average)/3;
			
			remaingSeconds=( Percents[Percents.length-1]-critical )*average;
			remaingSeconds=(remaingSeconds/1000);
			
			if(remaingSeconds<0){remaingSeconds=0;}
			
			ViewController.Time(remaingSeconds,true);
			this.seconds=remaingSeconds;
			
		}else{
			
			//estimating remaing time...
			ViewController.Time(0,false);
			
		}
		
	}
	
	this.reset=function(){
		
		Percents=[];
		Times=[];
		remaingSeconds=undefined;
		nativeRemaing=undefined;
		hidden=undefined;
		
	}
	
	
	 
	function currentTime(){
		var t=new Date();
		t=t.getTime();
		return(t);
	}
	
	
	function calcAverage(array){
		
		var n=array.length;
		var sum=0;
		
		for(var i=0;i<n;i++){
			sum=sum+array[i];
		}
		
		return(sum/n);
	}
	
	function calcAvgDiference(array){
		
		var n=array.length;
		var sum=0;
		
		for(var i=1;i<n;i++){
			sum=sum+(array[i]-array[i-1]);
		}
		
		return(sum/n);
	}
	
	function checkNativeRemaing(){
			if(System.Machine.PowerStatus.batteryCapacityRemaining>0){
				return(true);
			}else{
				return(false);
			}
	}

}
